"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getEnabledTypes = exports.RenamedWorkspaceTypeError = exports.DeprecatedTypesError = void 0;
const Data = __importStar(require("@effect/data/Data"));
const ReadonlyArray_1 = require("@effect/data/ReadonlyArray");
const Effect = __importStar(require("@effect/io/Effect"));
const is_array_of_strings_1 = require("tightrope/guard/is-array-of-strings");
const is_boolean_1 = require("tightrope/guard/is-boolean");
const is_empty_array_1 = require("tightrope/guard/is-empty-array");
const is_non_empty_string_1 = require("tightrope/guard/is-non-empty-string");
const constants_1 = require("../constants");
const name_and_version_props_1 = require("../strategy/name-and-version-props");
const versions_by_name_1 = require("../strategy/versions-by-name");
const get_custom_types_1 = require("./get-custom-types");
class DeprecatedTypesError extends Data.TaggedClass('DeprecatedTypesError') {
}
exports.DeprecatedTypesError = DeprecatedTypesError;
class RenamedWorkspaceTypeError extends Data.TaggedClass('RenamedWorkspaceTypeError') {
}
exports.RenamedWorkspaceTypeError = RenamedWorkspaceTypeError;
function getEnabledTypes({ cli, rcFile, }) {
    const deprecatedTypeProps = getDeprecatedTypeProps();
    if (deprecatedTypeProps.length > 0) {
        return Effect.fail(new DeprecatedTypesError({ types: deprecatedTypeProps }));
    }
    const allStrategiesByName = Object.fromEntries([
        ['dev', new versions_by_name_1.VersionsByNameStrategy('dev', 'devDependencies')],
        ['local', new name_and_version_props_1.NameAndVersionPropsStrategy('local', 'version', 'name')],
        ['overrides', new versions_by_name_1.VersionsByNameStrategy('overrides', 'overrides')],
        ['peer', new versions_by_name_1.VersionsByNameStrategy('peer', 'peerDependencies')],
        ['pnpmOverrides', new versions_by_name_1.VersionsByNameStrategy('pnpmOverrides', 'pnpm.overrides')],
        ['prod', new versions_by_name_1.VersionsByNameStrategy('prod', 'dependencies')],
        ['resolutions', new versions_by_name_1.VersionsByNameStrategy('resolutions', 'resolutions')],
        ...(0, get_custom_types_1.getCustomTypes)({ cli, rcFile }).map((customType) => [customType.name, customType]),
    ]);
    const allStrategyNames = Object.keys(allStrategiesByName);
    const names = {
        provided: ((0, is_non_empty_string_1.isNonEmptyString)(cli.types)
            ? cli.types.split(',')
            : (0, is_array_of_strings_1.isArrayOfStrings)(rcFile.dependencyTypes)
                ? rcFile.dependencyTypes
                : []).filter(is_non_empty_string_1.isNonEmptyString),
        enabled: [],
        positive: [],
        negative: [],
    };
    if ((0, is_empty_array_1.isEmptyArray)(names.provided) || names.provided.join('') === '**') {
        return Effect.succeed(allStrategyNames.map(getStrategyByName));
    }
    names.provided.forEach((name) => {
        if (name.startsWith('!')) {
            names.negative.push(name.replace('!', ''));
        }
        else {
            names.positive.push(name);
        }
    });
    if ((0, ReadonlyArray_1.isNonEmptyArray)(names.negative)) {
        allStrategyNames.forEach((name) => {
            if (!names.negative.includes(name)) {
                names.enabled.push(name);
            }
        });
    }
    if ((0, ReadonlyArray_1.isNonEmptyArray)(names.positive)) {
        names.positive.forEach((name) => {
            if (!names.enabled.includes(name)) {
                names.enabled.push(name);
            }
        });
    }
    if (names.enabled.includes('workspace')) {
        return Effect.fail(new RenamedWorkspaceTypeError({}));
    }
    return Effect.succeed(names.enabled.map(getStrategyByName));
    function getStrategyByName(type) {
        return allStrategiesByName[type];
    }
    /**
     * Look for dependency types defined using the old syntax of `{ prod: true }`
     * which was deprecated in syncpack@9.0.0.
     */
    function getDeprecatedTypeProps() {
        return constants_1.INTERNAL_TYPES.filter((key) => (0, is_boolean_1.isBoolean)(rcFile[key]));
    }
}
exports.getEnabledTypes = getEnabledTypes;
