"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setSemverRange = exports.getSemverRange = void 0;
const constants_1 = require("../constants");
const is_loose_semver_1 = require("../guards/is-loose-semver");
const is_semver_1 = require("../guards/is-semver");
const is_valid_semver_range_1 = require("../guards/is-valid-semver-range");
function getSemverRange(version) {
    if (version === '*')
        return version;
    const from1stNumber = version.search(/[0-9]/);
    const semverRange = version.slice(0, from1stNumber);
    return (0, is_valid_semver_range_1.isValidSemverRange)(semverRange) ? semverRange : '';
}
exports.getSemverRange = getSemverRange;
function setSemverRange(semverRange, version) {
    if (!(0, is_semver_1.isSemver)(version) || !(0, is_valid_semver_range_1.isValidSemverRange)(semverRange))
        return version;
    if (semverRange === '*')
        return semverRange;
    const nextVersion = (0, is_loose_semver_1.isLooseSemver)(version) ? version.replace(/\.x/g, '.0') : version;
    const from1stNumber = nextVersion.search(/[0-9]/);
    const from1stDot = nextVersion.indexOf('.');
    return semverRange === constants_1.RANGE.LOOSE
        ? `${nextVersion.slice(from1stNumber, from1stDot)}.x.x`
        : `${semverRange}${nextVersion.slice(from1stNumber)}`;
}
exports.setSemverRange = setSemverRange;
